import java.util.HashMap;

/**
 * Diese Klasse sammelt und liefert statistische Daten ber den
 * Zustand eines Feldes. Auf sehr flexible Weise: Es wird ein
 * Zhler angelegt und gepflegt fr jede Objektklasse, die im
 * Feld gefunden wird.
 * 
 * @author David J. Barnes und Michael Klling
 * @version 2006.03.30
 */
public class FeldStatistik
{
    // Die Zhler fr die jeweiligen Akteurstypen (Fuchs, Hase, etc.)
    // in der Simulation.
    private HashMap<Class, Zaehler> zaehler;
    // Sind die Zhlerstnde momentan aktuell?
    private boolean zaehlerAktuell;

    /**
     * Erzeuge ein FeldStatistik-Objekt.
     */
    public FeldStatistik()
    {
        // Wir legen eine Sammlung fr die Zhler an, die wir fr
        // die gefundenen Tierarten erzeugen.
        zaehler = new HashMap<Class, Zaehler>();
        zaehlerAktuell = true;
    }

    /**
     * Liefere Informationen ber die Bewohner im Feld.
     * @return Eine Beschreibung, welche Tiere das
     *         Feld bevlkern.
     */
    public String gibBewohnerInfo(Feld feld)
    {
        StringBuffer buffer = new StringBuffer();
        if(!zaehlerAktuell) {
            ermittleZaehlerstaende(feld);
        }
        for (Class schluessel : zaehler.keySet()) {
            Zaehler info = zaehler.get(schluessel);
            buffer.append(info.gibName());
            buffer.append(": ");
            buffer.append(info.gibStand());
            buffer.append(' ');
        }
        return buffer.toString();
    }
    
    /**
     * Verwerfe alle bisher gesammelten Daten; setze alle Zhler
     * auf Null zurck.
     */
    public void zuruecksetzen()
    {
        zaehlerAktuell = false;
        for(Class schluessel : zaehler.keySet()) {
            Zaehler einzelZaehler = zaehler.get(schluessel);
            einzelZaehler.zuruecksetzen();
        }
    }

    /**
     * Erhhe den Zhler fr eine Tierklasse.
     * @param tierklasse Klasse der Tierart, fr die erhht werden soll.
     */
    public void erhoeheZaehler(Class tierklasse)
    {
        Zaehler einzelZaehler = zaehler.get(tierklasse);
        if(einzelZaehler == null) {
            // Wir haben noch keinen Zhler fr
            // diese Spezies - also neu anlegen
            einzelZaehler = new Zaehler(tierklasse.getName());
            zaehler.put(tierklasse, einzelZaehler);
        }
        einzelZaehler.erhoehen();
    }

    /**
     * Signalisiere, dass eine Tierzhlung beendet ist.
     */
    public void zaehlungBeendet()
    {
        zaehlerAktuell = true;
    }

    /**
     * Stelle fest, ob die Simulation noch aktiv ist, also
     * ob sie weiterhin laufen sollte.
     * @return true wenn noch mehr als eine Spezies lebt.
     */
    public boolean istAktiv(Feld feld)
    {
        // Wieviele Zhler sind nicht Null.
        int nichtNull = 0;
        if(!zaehlerAktuell) {
            ermittleZaehlerstaende(feld);
        }
        for(Class schluessel : zaehler.keySet()) {
            Zaehler info = zaehler.get(schluessel);
            if(info.gibStand() > 0) {
                nichtNull++;
            }
        }
        return nichtNull > 1;
    }
    
    /**
     * Erzeuge Zhler fr die Anzahl der Fchse und Hasen.
     * Diese werden nicht stndig aktuell gehalten, whrend
     * Fchse und Hasen in das Feld gesetzt werden, sondern
     * jeweils bei der Abfrage der Zhlerstnde berechnet.
     * @param feld das Feld, fr das die Statistik erstellt
     *             werden soll.
     */
    private void ermittleZaehlerstaende(Feld feld)
    {
        zuruecksetzen();
        for(int zeile = 0; zeile < feld.gibTiefe(); zeile++) {
            for(int spalte = 0; spalte < feld.gibBreite(); spalte++) {
                Tier tier = feld.gibTierAn(zeile, spalte);
                if(tier != null) {
                    erhoeheZaehler(tier.getClass());
                }
            }
        }
        zaehlerAktuell = true;
    }
}
